package org.openhab.binding.openlicht.handler;

import static org.openhab.binding.openlicht.BindingConstants.*;

import java.math.BigDecimal;
import java.nio.ByteBuffer;
import java.util.concurrent.ScheduledExecutorService;

import javax.naming.ConfigurationException;

import org.eclipse.jdt.annotation.NonNull;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingStatusDetail;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.io.transport.mqtt.MqttBrokerConnection;
import org.eclipse.smarthome.io.transport.mqtt.MqttBrokersObserver;
import org.eclipse.smarthome.io.transport.mqtt.MqttException;
import org.eclipse.smarthome.io.transport.mqtt.MqttMessageSubscriber;
import org.eclipse.smarthome.io.transport.mqtt.MqttService;
import org.openhab.binding.openlicht.internal.ConfigurationHolder;
import org.osgi.framework.Version;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class AbstractMqttHandler extends BaseThingHandler
        implements MqttMessageSubscriber, MqttBrokersObserver {

    @NonNull
    protected final Logger logger = LoggerFactory.getLogger(AbstractMqttHandler.class);

    // protected OpenLichtConfiguration config;

    private @Nullable MqttService mqttService;

    private int usedTopicLength = 0;

    private MqttBrokerConnection currentBrokerConnection = null;

    private boolean warnNoBrokerConnection = true;

    protected @Nullable ScheduledExecutorService executor;

    private Version version;

    public AbstractMqttHandler(Thing thing, ConfigurationHolder configurationHolder) {
        super(thing);
        this.mqttService = configurationHolder.getMqttService();
        this.executor = configurationHolder.getExecutor();
        this.version = configurationHolder.getVersion();
    }

    @Override
    public void initialize() {
        // config = getConfigAs(OpenLichtConfiguration.class);
        String brokerName = getConfigValueAsString(CONFIG_BROKER_NAME);

        // remove this handler from mqttService (in case initialize is called because of a config update)
        this.mqttService.removeBrokersListener(this);
        this.mqttService.addBrokersListener(this);

        updateStatus(ThingStatus.UNKNOWN, ThingStatusDetail.CONFIGURATION_PENDING, "Searching broker");
        for (MqttBrokerConnection broker : this.mqttService.getAllBrokerConnections()) {
            brokerAdded(broker);
        }
        if (currentBrokerConnection == null) {
            MqttBrokerConnection myBroker = this.mqttService.getBrokerConnection(brokerName);
            if (myBroker != null) {
                brokerAdded(myBroker);
            }
        }
        publish("init version: " + version.toString());
    }

    protected String getConfigValueAsString(String key) {
        return (String) getThing().getConfiguration().get(key);
    }

    protected Integer getConfigValueAsInteger(String key) {
        BigDecimal bd = (BigDecimal) getThing().getConfiguration().get(key);
        return bd == null ? null : bd.intValue();
    }

    @Override
    public void brokerAdded(MqttBrokerConnection broker) {
        String brokerName = getConfigValueAsString(CONFIG_BROKER_NAME);
        if (broker.getName().equals(brokerName)) {
            // this is our broker!
            this.logger.info("Got correct broker, subscribing to topic {}", getTopic());
            currentBrokerConnection = broker;
            warnNoBrokerConnection = true;
            updateTopicLength();
            try {
                currentBrokerConnection.start();
                if (subscribeTopic()) {
                    broker.addConsumer(this);
                }
                updateStatus(ThingStatus.ONLINE);
                this.logger.info("Broker found, thing is online");
            } catch (MqttException e) {
                this.logger.error("Could not add subscriber to broker {}", brokerName);
                updateStatus(ThingStatus.OFFLINE, ThingStatusDetail.COMMUNICATION_ERROR,
                        "Could not add subscriber to broker " + brokerName);
            } catch (ConfigurationException e) {
                this.logger.error("Error during first connection to mqtt broker", e);
            }
        } else {
            this.logger.debug("Got another broker, not interessted!");
        }
    }

    private void updateTopicLength() {
        // if subscribed to subtopics, a "/#" is added
        // then, we want only the part up to the last "/"
        usedTopicLength = getTopic().length() - (subscribeSubTopics() ? 1 : 0);
    }

    @Override
    public void brokerRemoved(MqttBrokerConnection broker) {
        String brokerName = getConfigValueAsString(CONFIG_BROKER_NAME);
        if (broker.getName().equals(brokerName)) {
            // this was our broker!
            updateStatus(ThingStatus.OFFLINE, ThingStatusDetail.CONFIGURATION_PENDING, "Broker is offline");
            currentBrokerConnection = null;
        }
    }

    protected void publish(String message) {
        if (currentBrokerConnection == null) {
            if (warnNoBrokerConnection) {
                // just report once
                logger.warn("Can't publish message, no connection");
                warnNoBrokerConnection = false;
            }
            return;
        }
        String topic = subscribeSubTopics() ? (getTopic().substring(0, usedTopicLength) + "out")
                : (getTopic() + "/out");
        byte[] payload = message.getBytes();
        try {
            currentBrokerConnection.publish(topic, payload, null);
        } catch (MqttException e) {
            logger.error("Error while publishing", e);
        }
    }

    @Override
    public final String getTopic() {
        return getConfigValueAsString(CONFIG_BASE_TOPIC) + (getSubTopic() == null ? "" : "/" + getSubTopic())
                + (subscribeSubTopics() ? "/#" : "");
    }

    protected final int getTopicLength() {
        return this.usedTopicLength;
    }

    /**
     * Subclasses may override this to subscribe to a special subtopic.
     * Do not add a leading nor a trailing slash!
     *
     * @return <code>null</code> by default to not use subtopics
     */
    protected String getSubTopic() {
        return null;
    }

    /**
     * Subclasses may override this to prevent topic subscription
     *
     * @return <code>true</code> by default
     */
    protected boolean subscribeTopic() {
        return true;
    }

    /**
     * Subclasses may override this to also subscribe to any topic sharing the same prefix.
     * This means, that an additional "<code>/#</code>" is added at the end of the topic.
     *
     * @return <code>true</code> by default
     */
    protected boolean subscribeSubTopics() {
        return true;
    }

    /**
     * Writes a float array from the given input based on the length of the targeted output.
     *
     * @param input the buffer to read from
     * @param output the array to write in
     */
    protected void writeFloatArray(ByteBuffer input, float[] output) {
        for (int i = 0; i < output.length; i++) {
            output[i] = input.getFloat();
        }
    }

}
