/**
 * Copyright (c) 2014,2018 by the respective copyright holders.
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */
package org.openhab.binding.openlicht.handler;

import static org.openhab.binding.openlicht.BindingConstants.*;

import java.nio.ByteBuffer;
import java.nio.FloatBuffer;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.types.Command;
import org.openhab.binding.openlicht.internal.ConfigurationHolder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * The {@link AbstractSmartphoneHandler} is responsible for handling commands, which are
 * sent to one of the channels.
 *
 * @author René Schöne - Initial contribution
 */
public abstract class AbstractSmartphoneHandler extends AbstractMqttHandler {

    private final Logger logger = LoggerFactory.getLogger(AbstractSmartphoneHandler.class);

    private Set<String> seenUnsupportedCategories;

    public AbstractSmartphoneHandler(Thing thing, ConfigurationHolder configurationHolder) {
        super(thing, configurationHolder);
        seenUnsupportedCategories = new HashSet<>();
    }

    @Override
    public void handleCommand(ChannelUID channelUID, Command command) {
        String channelId = channelUID.getId();
        switch (channelId) {
            case CHANNEL_BRIGHTNESS:
                break;
            default:
                logger.warn("Unknown channel id {}!", channelId);
        }
    }

    @Override
    public void processMessage(String topic, byte[] payload) {
        // switch category of message (last part of topic after last slash)
        String category = topic.substring(getTopicLength());
        ByteBuffer buffer = ByteBuffer.wrap(payload);
        FloatBuffer floatBuffer;
        switch (category) {
            case MQTT_CATEGORY_BRIGHTNESS:
                updateState(CHANNEL_BRIGHTNESS, new DecimalType(buffer.getFloat()));
                break;
            case MQTT_CATEGORY_ROTATION:
                floatBuffer = buffer.asFloatBuffer();
                updateState(CHANNEL_ROTATION_X, new DecimalType(floatBuffer.get(0)));
                updateState(CHANNEL_ROTATION_Y, new DecimalType(floatBuffer.get(1)));
                updateState(CHANNEL_ROTATION_Z, new DecimalType(floatBuffer.get(2)));
            default:
                logUnsupportedCategory(category);
        }
    }

    private void logUnsupportedCategory(String category) {
        if (seenUnsupportedCategories.add(category)) {
            logger.warn("Category not supported: {}", category);

        }
    }

    @Override
    protected String getSubTopic() {
        return "samsung";
    }

}
